import argparse as ap
import os

class HtaccessParser:
    def __init__(self, logger=None):
        self.redirect_types = ['RewriteRule', 'RedirectMatch', 'Redirect']
        self.logger = logger

    def parse_line(self, line):
        res_line = ''

        line_parts = line.split()
        line_type = line_parts[0]

        if line_type in self.redirect_types:
            
            if line_type == 'Redirect':
                res_line = '{} {} [status={}]'.format(line_parts[2], line_parts[3], line_parts[1])

            elif line_type == 'RewriteRule':
                res_line = '{} {}'.format(line_parts[1], line_parts[2])

            elif line_type == 'RedirectMatch':
                res_line = '{} {}'.format(line_parts[2], line_parts[3])

        return res_line


class HtaccessReader:
    def __init__(self, logger=None):
        self.line_parser = HtaccessParser(logger=logger)
        if logger:
            self.logger = logger

    def read_htaccess(self, htaccess_file, map_name):
        abs_path = os.path.abspath(htaccess_file)
        res = []

        f = open(abs_path, 'r', encoding='UTF-8')

        for line in f:
            if len(line) > 2 and not line.startswith('#') and '#' not in line:
                res.append('\n')
                res.append('# {}'.format(line)) # for tests
                res.append(self.line_parser.parse_line(line))
                res.append('\n')

            

        f.close()

        self.write_map(res, map_name)


    def write_map(self, map_content, map_name):
        abs_path = os.path.abspath(map_name)

        f = open(abs_path, 'w')
        for line in map_content:
            f.write(line)
        f.close()


def main(args=None):
    parser = ap.ArgumentParser(description='Redirector: .htaccess converter')
    parser.add_argument('htaccess_file', metavar='H', type=str, nargs=1, help='.htaccess file that defines redirect rules')
    parser.add_argument('map_name', metavar='M', type=str, nargs=1, help='Output map name')

    reader = HtaccessReader()

    if not args:
        args = parser.parse_args()

    reader.read_htaccess(args.htaccess_file[0], args.map_name[0])


#if __name__ == "__main__":
#    main()

