from redirector.utils.const import MAPS_DIR, CONFIG_DIR

class Generator:
    def __init__(self):
        self.map_gen = MapGenerator()
        self.conf_gen = ConfigGenerator()
    
    def get_conf_dirs(self):
        return MAPS_DIR, CONFIG_DIR

    
    def generate(self, redirects_data, project_name):
        redirects_map = self.map_gen.generate_map(redirects_data[0], project_name)
        redirects_with_options = self.map_gen.generate_opt_map(redirects_data[1], project_name)
        conf_data = self.conf_gen.generate_config(project_name, [code for code, data in redirects_with_options])

        maps_dir, config_dir = self.get_conf_dirs()

        try:
            with open(f"{maps_dir}/{project_name}.map", "w") as map_file:
                map_file.write(redirects_map)
            for code, data in redirects_with_options:
                if code == "301":
                    code = "permanent"
                elif code == "302":
                    code = "redirect"
                with open(maps_dir + "/%s_%s_options.map" % (project_name, code), "w") as map_file:
                    map_file.write(data)
            with open(config_dir + "/%s.conf" % project_name, "w") as conf_file:
                conf_file.write(conf_data)
        except Exception as e:
            print(e)
            raise self.GenerationError("Can\'t generate new .conf or new .map file \
                 from %s .map file for %s project" % (map_file, project_name), e)
    
    class GenerationError(Exception):
        def __init__(self, message, errors):
            super().__init__(message)
            self.errors = errors



class MapGenerator:
    def __init__(self):
        self.start_line = "map $uri $%s_redirect {"
        self.opt_start_line = "map $uri $%s_%s_redirect {" # (project_name, redirect_option)
     
        self.endline = "\n}"

    def generate_map(self, redirects, project_name):
        data = self.start_line % project_name
        for arg1, arg2 in redirects:
            # print('redirect is', arg1, arg2)
            data += "\n\t/%s\t/%s;" % (project_name+arg1, project_name+arg2)
        data += self.endline
        return data
    
    def generate_opt_map(self, redirect_dict, project_name):
        result = []
        for code, redirects in redirect_dict.items():
            if code == "301":
                code = "permanent"
            elif code == "302":
                code = "redirect"
            data = self.opt_start_line % (project_name, code)
            for arg1, arg2 in redirects:
                data += "\n\t%s\t%s;" % (arg1, arg2)
            data += self.endline
            result.append((code, data))
        return result

            
            
class ConfigGenerator:
    def __init__(self):
        self.start_line = "if ($%s_redirect) {\n"
        self.opt_start_line = "if ($%s_%s_redirect) {\n"
        self.rewrite_line = "\trewrite ^/%s/(.*)$ $%s_redirect %s;\n"
        self.opt_rewrite_line = "\trewrite ^/%s/(.*)$ $%s_%s_redirect %s;\n"
        self.option_line = "return %s;\n}\n"

    def generate_config(self, project_name, options):
        data = (self.start_line % project_name) + (self.rewrite_line % (project_name, project_name, "redirect")) + "}\n"

        for code in options:
            if code == "permanent":
                data += (self.opt_start_line % (project_name, "permanent")) + (self.opt_rewrite_line % (project_name, project_name, "permanent", "permanent")) + "}\n"
            elif code == "redirect":
                data += (self.opt_start_line % (project_name, "redirect")) + (self.opt_rewrite_line % (project_name, project_name, "redirect", "redirect")) + "}\n"
            else:
                data += (self.opt_start_line % (project_name, code)) + (self.opt_rewrite_line % (project_name, project_name, code,  "redirect")) + (self.option_line % code)
        return data

